tree grammar FilterExpressionPredicateGen;

options {
tokenVocab=FilterExpression; 
ASTLabelType=CommonTree;
}

@header { 
package org.ait.table.filter; 

import com.google.common.base.Predicate;
import com.google.common.base.Predicates;
import org.ait.table.predicates.*;
import java.util.GregorianCalendar;
}

createPredicate	returns [Predicate pred]
	:	 p=predicate {$pred = p;}
	;

predicate returns [Predicate pred]
	: 'is.na' {$pred=Predicates.isNull();}
	| 'is.null' {$pred=Predicates.isNull();}
	| ^(OR a=predicate b=predicate) {$pred=Predicates.or(a,b);}
	| ^(AND a=predicate b=predicate){$pred=Predicates.and(a,b);}
	| ^(NOT p=predicate){$pred=Predicates.not(p);}
	| textPredicate {$pred=$textPredicate.pred;}
	| numericPredicate {$pred=$numericPredicate.pred;}
	| datePredicate {$pred=$datePredicate.pred;}
	;

textPredicate returns [Predicate pred]
	: ^('contains' e=STRING_LITERAL) {$pred = new Contains($e.text.substring(1,$e.text.length()-1));}
	| ^('equals'  TO? e=STRING_LITERAL) {$pred = new EqualString($e.text.substring(1,$e.text.length()-1));}
	;

numericPredicate returns [Predicate pred]
	: ^('='  number) {$pred = new Equal($number.value);}
	| ^('<'  number) {$pred = new Less($number.value);}
	| ^('<='  number) {$pred = new LessOrEqual($number.value);}
	| ^('>'  number) {$pred = new Greater($number.value);}
	| ^('>='  number) {$pred = new GreaterOrEqual($number.value);}
	| ^(TO a=number  b=number) {$pred = new BetweenTo(a,b);}
	;

number returns [Number value]
	: INTEGER {$value = new Integer($INTEGER.text);}
	| FLOAT {$value = new Double($FLOAT.text);}
	;

datePredicate returns [Predicate pred]
	: ^('='  (val=date|val=time)) {$pred = new Equal($val.value.getTimeInMillis());}
	| ^('<'  (val=date|val=time)) {$pred = new Less($val.value.getTimeInMillis());}
	| ^('<='  (val=date|val=time)) {$pred = new LessOrEqual($val.value.getTimeInMillis());}
	| ^('>'  (val=date|val=time)) {$pred = new Greater($val.value.getTimeInMillis());}
	| ^('>='  (val=date|val=time)) {$pred = new GreaterOrEqual($val.value.getTimeInMillis());}
	| ^(TO (val1=date|val1=time)  (val2=date|val2=time)) {$pred = new BetweenTo(val1.getTimeInMillis(),val2.getTimeInMillis());}
	;


time returns [GregorianCalendar value]
	:^(TIME date (hour=INTEGER | hour=DATENUM) (minute=INTEGER | minute=DATENUM) (second=INTEGER | second=DATENUM))
	  {
	   $date.value.set(GregorianCalendar.HOUR,Integer.parseInt($hour.text));
	   $date.value.set(GregorianCalendar.MINUTE,Integer.parseInt($minute.text));
	   $date.value.set(GregorianCalendar.SECOND,Integer.parseInt($second.text));
	   $value = $date.value;
	  };

date	returns [GregorianCalendar value]
	: ^(DATE (year=INTEGER | year=DATENUM) (month=INTEGER | month=DATENUM) (day=INTEGER | day=DATENUM)) 
		{$value = new GregorianCalendar(Integer.parseInt($year.text),Integer.parseInt($month.text),Integer.parseInt($day.text));}
	;

